@@ -16,5 +16,9 @@ __all__ = [
 		"extras_require",
 		]
 
-__version__ = "0.1.1"
-extras_require = {"readme": ["readme-renderer[md]>=27.0"], "all": ["readme-renderer[md]>=27.0"]}
+__version__ = "0.2.1"
+extras_require = {
+		"readme": ["docutils==0.16", "readme-renderer[md]>=27.0"],
+		"cli": ["click>=7.1.2", "consolekit>=1.1.2"],
+		"all": ["click>=7.1.2", "consolekit>=1.1.2", "docutils==0.16", "readme-renderer[md]>=27.0"]
+		}
