@@ -38,7 +38,7 @@ from dom_toml.encoder import _dump_str
 from dom_toml.parser import AbstractConfigParser, BadConfigError
 from domdf_python_tools.paths import PathPlus, in_directory
 from domdf_python_tools.typing import PathLike
-from domdf_python_tools.words import word_join
+from domdf_python_tools.words import Plural, word_join
 from packaging.markers import Marker
 from packaging.requirements import Requirement
 from packaging.specifiers import SpecifierSet
@@ -47,18 +47,20 @@ from packaging.version import Version
 # this package
 from pyproject_parser.classes import License, Readme
 from pyproject_parser.parsers import BuildSystemParser, PEP621Parser
-from pyproject_parser.type_hints import Author, BuildSystemDict, ContentTypes, ProjectDict
+from pyproject_parser.type_hints import Author, BuildSystemDict, ContentTypes, ProjectDict, _PyProjectAsTomlDict
 
 __author__: str = "Dominic Davis-Foster"
 __copyright__: str = "2021 Dominic Davis-Foster"
 __license__: str = "MIT License"
-__version__: str = "0.1.1"
+__version__: str = "0.2.1"
 __email__: str = "dominic@davis-foster.co.uk"
 
 __all__ = ["PyProject", "PyProjectTomlEncoder", "_PP"]
 
 _PP = TypeVar("_PP", bound="PyProject")
 
+_keys = Plural("key", "keys")
+
 
 class PyProjectTomlEncoder(dom_toml.TomlEncoder):
 	"""
@@ -74,8 +76,6 @@ class PyProjectTomlEncoder(dom_toml.TomlEncoder):
 		self.dump_funcs[Requirement] = self.dump_packaging_types
 		self.dump_funcs[Marker] = self.dump_packaging_types
 		self.dump_funcs[SpecifierSet] = self.dump_packaging_types
-		self.dump_funcs[Readme] = self.dump_readme
-		self.dump_funcs[License] = self.dump_license
 
 	@staticmethod
 	def dump_packaging_types(obj: Union[Version, Requirement, Marker, SpecifierSet]) -> str:
@@ -89,28 +89,6 @@ class PyProjectTomlEncoder(dom_toml.TomlEncoder):
 
 		return _dump_str(str(obj))
 
-	def dump_readme(self, obj: Readme) -> str:
-		"""
-		Convert a :class:`~.Readme` to TOML.
-
-		:param obj:
-		"""
-
-		readme_dict = obj.to_pep621_dict()
-		if set(readme_dict.keys()) == {"file"}:
-			return _dump_str(readme_dict["file"])
-
-		return self.dump_inline_table(readme_dict)
-
-	def dump_license(self, obj: License) -> str:
-		"""
-		Convert a :class:`~.License` to TOML.
-
-		:param obj:
-		"""
-
-		return self.dump_inline_table(obj.to_pep621_dict())
-
 
 @serde
 @attr.s
@@ -195,7 +173,10 @@ class PyProject:
 						tool_table[tool_name] = cls.tool_parsers[tool_name].parse(tool_subtable)
 
 		if keys:
-			raise BadConfigError(f"Unexpected top level keys: {word_join(sorted(keys), use_repr=True)}")
+			raise BadConfigError(
+					f"Unexpected top level {_keys(len(keys))}: "
+					f"{word_join(sorted(keys), use_repr=True)}",
+					)
 
 		return cls(
 				build_system=build_system_table,
@@ -215,7 +196,26 @@ class PyProject:
 
 		# TODO: filter out default values (lists and dicts)
 
-		toml_dict = {"build-system": self.build_system, "project": self.project, "tool": self.tool}
+		toml_dict: _PyProjectAsTomlDict = {
+				"build-system": self.build_system, "project": self.project, "tool": self.tool
+				}
+
+		if toml_dict["project"] is not None:
+			if "license" in toml_dict["project"] and toml_dict["project"]["license"] is not None:
+				toml_dict["project"] = {  # type: ignore
+					**toml_dict["project"],  # type: ignore
+					"license": toml_dict["project"]["license"].to_pep621_dict()
+					}
+
+		if toml_dict["project"] is not None:
+			if "readme" in toml_dict["project"] and toml_dict["project"]["readme"] is not None:
+				readme_dict = toml_dict["project"]["readme"].to_pep621_dict()
+
+				if set(readme_dict.keys()) == {"file"}:
+					toml_dict["project"] = {**toml_dict["project"], "readme": readme_dict["file"]}  # type: ignore
+				else:
+					toml_dict["project"] = {**toml_dict["project"], "readme": readme_dict}  # type: ignore
+
 		return dom_toml.dumps(toml_dict, encoder)
 
 	def dump(
@@ -238,23 +238,29 @@ class PyProject:
 		return as_toml
 
 	@classmethod
-	def reformat(cls: Type[_PP], filename: PathLike) -> str:
+	def reformat(
+			cls: Type[_PP],
+			filename: PathLike,
+			encoder: Union[Type[toml.TomlEncoder], toml.TomlEncoder, None] = PyProjectTomlEncoder,
+			) -> str:
 		"""
 		Reformat the given ``pyproject.toml`` file.
 
 		:param filename: The file to reformat.
+		:param encoder: The :class:`toml.TomlEncoder` to use for constructing the output string.
 
 		:returns: A string containing the reformatted TOML.
+
+		.. versionchanged:: 0.2.0
+
+			* Added the ``encoder`` argument.
+			* The parser configured as :attr:`~.project_table_parser` is now used to parse
+			  the ``[project]`` table, rather than always using :class:`~.PEP621Parser`.
+
 		"""
 
-		original_project_table_parser = cls.project_table_parser
-		cls.project_table_parser = PEP621Parser()
-
-		try:
-			config = cls.load(filename, set_defaults=False)
-			return config.dump(filename)
-		finally:
-			cls.project_table_parser = original_project_table_parser
+		config = cls.load(filename, set_defaults=False)
+		return config.dump(filename, encoder=encoder)
 
 	def resolve_files(self):
 		"""
