@@ -27,17 +27,16 @@ TOML configuration parsers.
 #
 
 # stdlib
+import collections.abc
 import os
 import re
 from abc import ABCMeta
 from collections import defaultdict
-from typing import Any, ClassVar, Dict, Iterable, List, Sequence, Union, cast
+from typing import Any, Callable, ClassVar, Dict, Iterable, List, Union, cast
 
 # 3rd party
 from apeye import URL
 from dom_toml.parser import TOML_TYPES, AbstractConfigParser, BadConfigError, construct_path
-from domdf_python_tools.paths import PathPlus
-from domdf_python_tools.typing import PathLike
 from email_validator import EmailSyntaxError, validate_email  # type: ignore
 from natsort import natsorted, ns
 from packaging.specifiers import InvalidSpecifier, Specifier, SpecifierSet
@@ -87,7 +86,7 @@ class RequiredKeysConfigParser(AbstractConfigParser, metaclass=ABCMeta):
 			if key in config:
 				continue
 			elif set_defaults and (key in self.defaults or key in self.factories):
-				continue
+				continue  # pragma: no cover https://github.com/nedbat/coveragepy/issues/198
 			else:
 				raise BadConfigError(f"The {construct_path([self.table_name, key])!r} field must be provided.")
 
@@ -111,11 +110,11 @@ class RequiredKeysConfigParser(AbstractConfigParser, metaclass=ABCMeta):
 		if isinstance(obj, str):
 			name = construct_path(path)
 			raise TypeError(
-					f"Invalid {what} type for {name!r}: "
+					f"Invalid {what} for {name!r}: "
 					f"expected <class 'collections.abc.Sequence'>, got {type(obj)!r}",
 					)
 
-		self.assert_type(obj, Sequence, path, what=what)
+		self.assert_type(obj, collections.abc.Sequence, path, what=what)
 
 
 class BuildSystemParser(RequiredKeysConfigParser):
@@ -123,15 +122,11 @@ class BuildSystemParser(RequiredKeysConfigParser):
 	Parser for the ``[build-system]`` table from ``pyproject.toml``.
 	"""
 
-	table_name = "build-system"
-	required_keys = ["requires"]
-	keys = [
-			"requires",
-			"build-backend",
-			"backend-path",
-			]
-	defaults = {"build-backend": None, "backend-path": None}
-	factories = {"requires": list}
+	table_name: ClassVar[str] = "build-system"
+	required_keys: ClassVar[List[str]] = ["requires"]
+	keys: ClassVar[List[str]] = ["requires", "build-backend", "backend-path"]
+	factories: ClassVar[Dict[str, Callable[..., Any]]] = {"requires": list}
+	defaults: ClassVar[Dict[str, Any]] = {"build-backend": None, "backend-path": None}
 
 	def parse_requires(self, config: Dict[str, TOML_TYPES]) -> List[ComparableRequirement]:
 		"""
@@ -180,11 +175,20 @@ class BuildSystemParser(RequiredKeysConfigParser):
 
 		return parsed_backend_paths
 
-	def parse(  # type: ignore[override]  # noqa: D102
+	def parse(  # type: ignore[override]
 		self,
 		config: Dict[str, TOML_TYPES],
 		set_defaults: bool = False,
 		) -> BuildSystemDict:
+		"""
+		Parse the TOML configuration.
+
+		:param config:
+		:param set_defaults: If :py:obj:`True`, the values in
+			:attr:`dom_toml.parser.AbstractConfigParser.defaults` and
+			:attr:`dom_toml.parser.AbstractConfigParser.factories`
+			will be set as defaults for the returned mapping.
+		"""
 
 		parsed_config = super().parse(config, set_defaults)
 
@@ -206,7 +210,7 @@ class PEP621Parser(RequiredKeysConfigParser):
 	Parser for :pep:`621` metadata from ``pyproject.toml``.
 	"""
 
-	table_name = "project"
+	table_name: ClassVar[str] = "project"
 	keys: List[str] = [
 			"name",
 			"version",
@@ -225,15 +229,15 @@ class PEP621Parser(RequiredKeysConfigParser):
 			"dependencies",
 			"optional-dependencies",
 			]
-	required_keys = ["name"]
-	defaults = {
+	required_keys: ClassVar[List[str]] = ["name"]
+	defaults: ClassVar[Dict[str, Any]] = {
 			"version": None,
 			"description": None,
 			"readme": None,
 			"requires-python": None,
 			"license": None,
 			}
-	factories = {
+	factories: ClassVar[Dict[str, Callable[..., Any]]] = {
 			"authors": list,
 			"maintainers": list,
 			"keywords": list,
@@ -416,7 +420,7 @@ class PEP621Parser(RequiredKeysConfigParser):
 				try:
 					email = validate_email(email, check_deliverability=False).email
 				except EmailSyntaxError as e:
-					raise BadConfigError(f"Invalid email {email}: {e} ")
+					raise BadConfigError(f"Invalid email {email!r}: {e} ")
 
 			all_authors.append({"name": name, "email": email})
 
@@ -570,8 +574,10 @@ class PEP621Parser(RequiredKeysConfigParser):
 
 		return sorted(combine_requirements(parsed_dependencies))
 
-	@staticmethod
-	def parse_optional_dependencies(config: Dict[str, TOML_TYPES]) -> Dict[str, List[ComparableRequirement]]:
+	def parse_optional_dependencies(
+			self,
+			config: Dict[str, TOML_TYPES],
+			) -> Dict[str, List[ComparableRequirement]]:
 		"""
 		Parse the
 		`optional-dependencies <https://www.python.org/dev/peps/pep-0621/#dependencies-optional-dependencies>`_ table.
@@ -589,14 +595,16 @@ class PEP621Parser(RequiredKeysConfigParser):
 		optional_dependencies = config["optional-dependencies"]
 
 		if not isinstance(optional_dependencies, dict):
-			raise TypeError(err_template.format('', type(optional_dependencies)))
+			raise TypeError(err_template.format(idx_string='', actual_type=type(optional_dependencies)))
 
 		for extra, dependencies in optional_dependencies.items():
+			self.assert_sequence_not_str(dependencies, path=["project", "optional-dependencies", extra])
+
 			for idx, dep in enumerate(dependencies):
 				if isinstance(dep, str):
 					parsed_optional_dependencies[extra].add(ComparableRequirement(dep))
 				else:
-					raise TypeError(err_template.format(f'{extra}[{idx}]', type(dep)))
+					raise TypeError(err_template.format(idx_string=f'.{extra}[{idx}]', actual_type=type(dep)))
 
 		return {e: sorted(combine_requirements(d)) for e, d in parsed_optional_dependencies.items()}
 
